/* GENERAL DEFINITIONS ****************************************************/

#include <internal/hal/irq.h>

#include <ddk/kedef.h> 
#include <ddk/iodef.h>

/*
 * PURPOSE: Number of a thread priority levels
 */
#define NR_PRIORITY_LEVELS (32)

/*
 * PURPOSE: Type of queue to insert a work item in
 */
enum
{
  CriticalWorkQueue,
  DelayedWorkQueue,
  HyperCriticalWorkQueue,
};

/*
 * Types of memory to allocate
 */
enum
{
   NonPagedPool,
   NonPagedPoolMustSucceed,
   NonPagedPoolCacheAligned,
   NonPagedPoolCacheAlignedMustS,
   PagedPool,
   PagedPoolCacheAligned,
};

/*
 * Possible status codes
 * FIXME: These may not be the actual values used by NT
 */
enum
{
   STATUS_SUCCESS = 0x0,
   STATUS_INSUFFICIENT_RESOURCES = 0x80000000,
   STATUS_OBJECT_NAME_EXISTS,
   STATUS_OBJECT_NAME_COLLISION,
//   STATUS_DATATYPE_MISALIGNMENT,
   STATUS_CTL_FILE_NOT_SUPPORTED,
//   STATUS_ACCESS_VIOLATION,
   STATUS_PORT_ALREADY_SET,
   STATUS_SECTION_NOT_IMAGE,
   STATUS_BAD_WORKING_SET_LIMIT,
   STATUS_INCOMPATIBLE_FILE_MAP,
   STATUS_HANDLE_NOT_WAITABLE,
   STATUS_PORT_DISCONNECTED,
   STATUS_NOT_LOCKED,
   STATUS_NOT_MAPPED_VIEW,
   STATUS_UNABLE_TO_FREE_VM,
   STATUS_UNABLE_TO_DELETE_SECTION,
   STATUS_MORE_PROCESSING_REQUIRED,
   STATUS_INVALID_CID,
   STATUS_BAD_INITIAL_STACK,
   STATUS_INVALID_VOLUME_LABEL,
   STATUS_SECTION_NOT_EXTENDED,
   STATUS_NOT_MAPPED_DATA,
   STATUS_INFO_LENGTH_MISMATCH,
   STATUS_INVALID_INFO_CLASS,
   STATUS_SUSPEND_COUNT_EXCEEDED,
   STATUS_NOTIFY_ENUM_DIR,
   STATUS_REGISTRY_RECOVERED,
   STATUS_REGISTRY_IO_FAILED,
   STATUS_KEY_DELETED,
   STATUS_NO_LOG_SPACE,
   STATUS_KEY_HAS_CHILDREN,
   STATUS_CHILD_MUST_BE_VOLATILE,
   STATUS_REGISTRY_CORRUPT,
   STATUS_DLL_NOT_FOUND,
   STATUS_DLL_INIT_FAILED,
   STATUS_ORDINAL_NOT_FOUND,
   STATUS_ENTRYPOINT_NOT_FOUND,
//   STATUS_PENDING,
   STATUS_MORE_ENTRIES,
//   STATUS_INTEGER_OVERFLOW,
   STATUS_BUFFER_OVERFLOW,
   STATUS_NO_MORE_FILES,
   STATUS_NO_INHERITANCE,
   STATUS_NO_MORE_EAS,
   STATUS_NO_MORE_ENTRIES,
   STATUS_GUIDS_EXHAUSTED,
   STATUS_AGENTS_EXHAUSTED,
   STATUS_UNSUCCESSFUL,
   STATUS_NOT_IMPLEMENTED,
   STATUS_ILLEGAL_FUNCTION,
//   STATUS_IN_PAGE_ERROR,
   STATUS_PAGEFILE_QUOTA,
   STATUS_COMMITMENT_LIMIT,
   STATUS_SECTION_TOO_BIG,
   RPC_NT_SS_IN_NULL_CONTEXT,
   RPC_NT_INVALID_BINDING,
//   STATUS_INVALID_HANDLE,
   STATUS_OBJECT_FILE_MISMATCH,
   STATUS_FILE_CLOSED,
   STATUS_INVALID_PORT_HANDLE,
   STATUS_NOT_COMMITTED,
   STATUS_INVALID_PARAMETER,
   STATUS_INVALID_PARAMETER_1,
   STATUS_INVALID_PARAMETER_2,
   STATUS_INVALID_PARAMETER_3,
   STATUS_INVALID_PARAMETER_4,
   STATUS_INVALID_PARAMETER_5,
   STATUS_INVALID_PARAMETER_6,
   STATUS_INVALID_PARAMETER_7,
   STATUS_INVALID_PARAMETER_8,
   STATUS_INVALID_PARAMETER_9,
   STATUS_INVALID_PARAMETER_10,
   STATUS_INVALID_PARAMETER_11,
   STATUS_INVALID_PARAMETER_12,
   STATUS_INVALID_PARAMETER_MAX,
   STATUS_INVALID_PAGE_PROTECTION,
   STATUS_RESOURCE_DATA_NOT_FOUND,
   STATUS_RESOURCE_TYPE_NOT_FOUND,
   STATUS_RESOURCE_NAME_NOT_FOUND,
   STATUS_RESOURCE_LANG_NOT_FOUND,
   STATUS_NO_SUCH_DEVICE,
   STATUS_NO_SUCH_FILE,
   STATUS_INVALID_DEVICE_REQUEST,
   STATUS_END_OF_FILE,
   STATUS_FILE_FORCED_CLOSED,
   STATUS_WRONG_VOLUME,
   STATUS_NO_MEDIA,
   STATUS_NO_MEDIA_IN_DEVICE,
   STATUS_NONEXISTENT_SECTOR,
   STATUS_WORKING_SET_QUOTA,
//   STATUS_NO_MEMORY,
   STATUS_CONFLICTING_ADDRESS,
   STATUS_INVALID_SYSTEM_SERVICE,
   STATUS_THREAD_IS_TERMINATING,
   STATUS_PROCESS_IS_TERMINATING,
   STATUS_INVALID_LOCK_SEQUENCE,
   STATUS_INVALID_VIEW_SIZE,
   STATUS_ALREADY_COMMITTED,
   STATUS_ACCESS_DENIED,
   STATUS_FILE_IS_A_DIRECTORY,
   STATUS_CANNOT_DELETE,
   STATUS_INVALID_COMPUTER_NAME,
   STATUS_FILE_DELETED,
   STATUS_DELETE_PENDING,
   STATUS_PORT_CONNECTION_REFUSED,
   STATUS_NO_SUCH_PRIVILEGE,
   STATUS_PRIVILEGE_NOT_HELD,
   STATUS_CANNOT_IMPERSONATE,
   STATUS_LOGON_FAILURE,
   STATUS_ACCOUNT_RESTRICTION,
   STATUS_INVALID_LOGON_HOURS,
   STATUS_INVALID_WORKSTATION,
   STATUS_BUFFER_TOO_SMALL,
   STATUS_UNABLE_TO_DECOMMIT_VM,
   STATUS_DISK_CORRUPT_ERROR,
   STATUS_OBJECT_NAME_INVALID,
   STATUS_OBJECT_NAME_NOT_FOUND,
//   STATUS_OBJECT_NAME_COLLISION,
   STATUS_OBJECT_PATH_INVALID,
   STATUS_OBJECT_PATH_NOT_FOUND,
   STATUS_DFS_EXIT_PATH_FOUND,
   STATUS_OBJECT_PATH_SYNTAX_BAD,
   STATUS_DATA_OVERRUN,
   STATUS_DATA_LATE_ERROR,
   STATUS_DATA_ERROR,
   STATUS_CRC_ERROR,
   STATUS_SHARING_VIOLATION,
   STATUS_QUOTA_EXCEEDED,
   STATUS_MUTANT_NOT_OWNED,
   STATUS_SEMAPHORE_LIMIT_EXCEEDED,
   STATUS_DISK_FULL,
   STATUS_LOCK_NOT_GRANTED,
     
   STATUS_DEVICE_NOT_READY,
   STATUS_IO_TIMEOUT,
   STATUS_MEDIA_WRITE_PROTECTED,
   STATUS_NO_MEDIA_IN_DRIVE,
   STATUS_VERIFY_REQUIRED,
   STATUS_UNRECOGNIZED_MEDIA,
   STATUS_UNRECOGNIZED_VOLUME,
//   STATUS_WRONG_VOLUME,
     STATUS_FS_DRIVER_REQUIRED,
};
   
/*
 * This is a list of bug check types (not MS's)
 */
enum
{
   APC_INDEX_MISMATCH = 1,
   DEVICE_QUEUE_NOT_BUSY,
   INVALID_AFFINITY_SET,
   INVALID_DATA_ACCESS_TRAP,
   INVALID_PROCESS_ATTACH_ATTEMPT,
   INVALID_PROCESS_DEATTACH_ATTEMPT,
   INVALID_SOFTWARE_INTERRUPT,
   IRQL_NOT_DISPATCH_LEVEL,
   IRQL_NOT_GREATER_OR_EQUAL,
   NO_EXCEPTION_HANDLING_SUPPORT,
   MAXIMUM_WAIT_OBJECTS_EXCEEDED,
   MUTEX_LEVEL_NUMBER_VIOLATION,
   NO_USER_MODE_CONTEXT,
   SPIN_LOCK_ALREADY_OWNED,
   SPIN_LOCK_NOT_OWNED,
   THREAD_NOT_MUTEX_OWNER,
   TRAP_CAUSE_UNKNOWN,
   EMPTY_THREAD_REAPER_LIST,
   CREATE_DELETE_LOCK_NOT_LOCKED,
   LAST_CHANCE_CALLED_FROM_KMODE,
   CID_HANDLE_CREATION,
   CID_HANDLE_DELETION,
   REFERENCE_BY_POINTER,
   BAD_POOL_HEADER,
   MEMORY_MANAGMENT,
   PFN_SHARE_COUNT,
   PFN_REFERENCE_COUNT,
   NO_SPIN_LOCK_AVAILABLE,
   KMODE_EXCEPTION_NOT_HANDLED,
   SHARED_RESOURCE_CONV_ERROR,
   KERNEL_APC_PENDING_DURING_EXIT,
   QUOTA_UNDERFLOW,
   FILE_SYSTEM,
   FAT_FILE_SYSTEM,
   NTFS_FILE_SYSTEM,
   NPFS_FILE_SYSTEM,
   CDFS_FILE_SYSTEM,
   RDR_FILE_SYSTEM,
   CORRUPT_ACCESS_TOKEN,
   SECURITY_SYSTEM,
   INCONSISTENT_IRP,
   PANIC_STACK_SWITCH,
   PORT_DRIVER_INTERNAL,
   SCSI_DISK_DRIVER_INTERNAL,
   INSTRUCTION_BUS_ERROR,
   SET_OF_INVALID_CONTEXT,
   PHASE0_INITIALIZATION_FAILED,
   PHASE1_INITIALIZATION_FAILED,
   UNEXPECTED_INITIALIZATION_CALL,
   CACHE_MANAGER,
   NO_MORE_IRP_STACK_LOCATIONS,
   DEVICE_REFERENCE_COUNT_NOT_ZERO,
   FLOPPY_INTERNAL_ERROR,
   SERIAL_DRIVER_INTERNAL,
   SYSTEM_EXIT_OWNED_MUTEX,
   SYSTEM_UNWIND_PREVIOUS_USER,
   SYSTEN_SERVICE_EXCEPTION,
   INTERRUPT_UNWIND_ATTEMPTED,
   INTERRUPT_EXCEPTION_NOT_HANDLED,
   MULTIPROCESSOR_CONFIGURATION_NOT_SUPPORTED,
   NO_MORE_SYSTEM_PTES,
   TARGET_MDL_TOO_SMALL,
   MUST_SUCCEED_POOL_EMPTY,
   ATDISK_DRIVER_INTERNAL,
   NO_SUCH_PARTITION,
   MULTIPLE_IRP_COMPLETE_REQUESTS,
   INSUFFICENT_SYSTEM_MAP_PAGES,
   DEREF_UNKNOWN_LOGON_SERVICE,
   REF_UNKNOWN_LOGON_SERVICE,
   CANCEL_STATE_IN_COMPLETED_IRP,
   PAGE_FAULT_WITH_INTERRUPTS_OFF,
   IRQL_GT_ZERO_AT_SYSTEM_SERVICE,
   STREAMS_INTERNAL_ERROR,
   FATAL_UNHANDLED_HARD_ERROR,
   NO_PAGES_AVAILABLE,
   PFN_LIST_CORRUPT,
   NDIS_INTERNAL_ERROR,
   PAGE_FAULT_IN_NONPAGED_AREA,
   REGISTRY_ERROR,
   MAILSLOT_FILE_SYSTEM,
   NO_BOOT_DEVICE,
   LM_SERVER_INTERNAL_ERROR,
   DATA_COHERENCY_EXCEPTION,
   INSTRUCTION_COHERENCY_EXCEPTION,
   XNS_INTERNAL_ERROR,
   FTDISK_INTERNAL_ERROR,
   PINBALL_FILE_SYSTEM,
   CRITICAL_SERVICE_FAILED,
   SET_ENV_VAR_FAILED,
   HAL_INITIALIZED_FAILED,
   UNSUPPORTED_PROCESSOR,
   OBJECT_INITIALIZATION_FAILED,
   SECURITY_INITIALIZATION_FAILED,
   PROCESS_INITIALIZATION_FAILED,
   HAL1_INITIALIZATION_FAILED,   
};
enum
{
   KBUG_NONE,
   KBUG_ORPHANED_IRP,
   KBUG_IO_STACK_OVERFLOW,
   KBUG_OUT_OF_MEMORY,
   KBUG_POOL_FREE_LIST_CORRUPT,
     
   /*
    * These are well known but the actual value is unknown
    */
//   NO_PAGES_AVAILABLE,
     
   /*
    * These are well known (MS) bug types
    * (Reference: NT Insider 1997 - http://www.osr.com)
    */
   IRQL_NOT_LESS_OR_EQUAL = 0xa,
//   KMODE_EXCEPTION_NOT_HANDLED = 0x1e,
   UNEXPECTED_KERNEL_MODE_TRAP = 0x7f,
   PAGE_FAULT_IN_NON_PAGED_AREA = 0x50,
};

/*
 * PURPOSE: Object attributes
 */
enum
{
   OBJ_INHERIT = 0x1,
   OBJ_PERMANENT = 0x2,
   OBJ_EXCLUSIVE = 0x4,
   OBJ_CASE_INSENSITIVE = 0x8,
   OBJ_OPENIF = 0x10,
};

/*
 * PURPOSE: DPC priorities
 */
enum
{
   High,
   Medium,
   Low,
};

/*
 * PURPOSE: Timer types
 */
enum
  {
      NotificationTimer,
      SynchronizationTimer,	  
  };
   
/*
 * PURPOSE: Some drivers use these
 */
#define IN
#define OUT
#define OPTIONAL

/*
 * PURPOSE: Power IRP minor function numbers
 */
enum
{
   IRP_MN_QUERY_POWER,
   IRP_MN_SET_POWER,
   IRP_MN_WAIT_WAKE,
   IRP_MN_QUERY_CAPABILITIES,
   IRP_MN_POWER_SEQUENCE,
};

/*
 * PURPOSE: Used all over
 */
enum
{
   KernelMode,
   UserMode,
};
   
/*
 * PURPOSE: Arguments to MmProbeAndLockPages
 */
enum
{
   IoReadAccess,
   IoWriteAccess,
   IoModifyAccess,
};

#define MAXIMUM_VOLUME_LABEL_LENGTH (32)

/*
 * IRQ levels
 */
enum
{
   PASSIVE_LEVEL,
     
   /*
    * Which order for these (only DISPATCH_LEVEL is important for now)
    */
   APC_LEVEL,
   DISPATCH_LEVEL,
     
   /*
    * Above here are device specific IRQ levels
    */
   FIRST_DEVICE_SPECIFIC_LEVEL,
   HIGH_LEVEL = FIRST_DEVICE_SPECIFIC_LEVEL + NR_DEVICE_SPECIFIC_LEVELS,
};
  
